from collections.abc import AsyncIterable
from functools import lru_cache
from typing import Annotated

from fastapi import Depends
from sqlalchemy.ext.asyncio import (
    AsyncEngine,
    AsyncSession,
    async_sessionmaker,
    create_async_engine,
)

from fastfood_two.common.stub import Stub
from fastfood_two.config import Config


@lru_cache()
def create_engine(
    config: Annotated[Config, Depends(Stub(Config))],
) -> AsyncEngine:
    return create_async_engine(config.db.url)


def create_session_maker(
    engine: Annotated[AsyncEngine, Depends(Stub(AsyncEngine))],
) -> async_sessionmaker[AsyncSession]:
    return async_sessionmaker(engine, expire_on_commit=False)


async def get_session(
    session_maker: Annotated[
        async_sessionmaker[AsyncSession],
        Depends(Stub(async_sessionmaker[AsyncSession])),
    ],
) -> AsyncIterable[AsyncSession]:
    async with session_maker() as session:
        yield session
